//
//  FX2USB.m
//  WaveControl
//
//  Created by Andrew Stagg on 20/04/07.
//  Copyright 2007 Andrew Stagg. All rights reserved.
//

#import "FX2USB.h"

char							*fnamefirmware;
char							*fnameloader;
char							*fnamevhdl;
static IONotificationPortRef	gNotifyPort;
static io_iterator_t			addedIter;
static io_iterator_t			removedIter;
IOUSBInterfaceInterface			**curinterface;
int								downloadpipe;
int								uploadpipe;
int								fsm;
int								ready;
int								laser_state;
unsigned long					status;

id *guiControl;

@implementation FX2USB

- (id)initWithController:(id)sender andFirmware:(NSString*)firmware andLoader:(NSString *)loader andVHDL:(NSString*)vhdl
{
	[super init];
	
	guiControl = sender;
	fnamefirmware = (char *)[firmware cString];
	fnameloader = (char *)[loader cString];
	fnamevhdl = (char *)[vhdl cString];
	fsm = 0;
	laser_state = 0;
	curinterface = nil;
	AcquireDevice();
	
	return self;
}

- (int)sendPlotter:(int)command
{
	return send_plotter(command);
}

- (void)dealloc
{
	usbdone();
	[super dealloc];
}

IOReturn ConfigureDevice(IOUSBDeviceInterface **dev)
{
    UInt8                           numConfig;
    IOReturn                        kr;
    IOUSBConfigurationDescriptorPtr configDesc;
	
    // Get the number of configurations
    kr = (*dev)->GetNumberOfConfigurations(dev, &numConfig);
    if (!numConfig)
        return -1;
	
    // Get the configuration descriptor for index 0
    kr = (*dev)->GetConfigurationDescriptorPtr(dev, 0, &configDesc);
    if (kr) {
        NSLog(@"Could not get configuration descriptor for index 0 (err = %08x)", kr);
        return -1;
    }
	
    // Set the device’s configuration
    kr = (*dev)->SetConfiguration(dev, configDesc->bConfigurationValue);
    if (kr) {
        NSLog(@"Could not set configuration to value 0 (err = %08x)", kr);
        return -1;
    }
    return kIOReturnSuccess;
}

/*
 * Writes each structure to appropriate address on device (Cypress FX2)
 */
IOReturn WriteToDevice(IOUSBDeviceInterface **dev, UInt16 deviceAddress, UInt16 length, UInt8 writeBuffer[]) 
{ 
	IOUSBDevRequest request; 
	request.bmRequestType = USBmakebmRequestType(kUSBOut, kUSBVendor, kUSBDevice); 
	request.bRequest = 0xa0; 
	request.wValue = deviceAddress; 
	request.wIndex = 0; 
	request.wLength = length; 
	request.pData = writeBuffer;
	
	return (*dev)->DeviceRequest(dev, &request);
}

/*
 * Reads each structure back from address on device (Cypress FX2)
 */
IOReturn ReadFromDevice(IOUSBDeviceInterface **dev, UInt16 deviceAddress, UInt16 length, UInt8 readBuffer[]) 
{ 
	IOUSBDevRequest request;
	request.bmRequestType = USBmakebmRequestType(kUSBIn, kUSBVendor, kUSBDevice); 
	request.bRequest = 0xa0; 
	request.wValue = deviceAddress; 
	request.wIndex = 0; 
	request.wLength = length; 
	request.pData = readBuffer;
	
	return (*dev)->DeviceRequest(dev, &request); 
}

/*
 * Prepare device to receive download (Cypress FX2)
 */
IOReturn DownloadToDevice(IOUSBDeviceInterface **dev, char *fname)
{
	UInt8				writeVal;
	IOReturn			kr;
	FILE				*hexFile;
	INTEL_HEX_RECORD	anchorCode;
	INTEL_HEX_RECORD	anchorRead;
	int					i;
	
	hexFile = fopen(fname, "r");
    if (hexFile == nil) {
        NSLog(@"File open failed");
        return kIOReturnNotOpen;
    }
	
	// assert reset (tells device to expect download)
	writeVal = 1; // 1 indicates download on FX2
	kr = WriteToDevice(dev, k8051_CPUCS, 1, &writeVal);
	if (kr != kIOReturnSuccess) { 
		NSLog(@"Error: WriteToDevice reset returned err 0x%x", kr); 
		(*dev)->USBDeviceClose(dev); 
		(*dev)->Release(dev); 
		return kr; 
	}
	
	// download firmware
	while (1) {
		kr = hexRead(&anchorCode, hexFile);
		if (anchorCode.Type != 0)
            break;        
		kr = WriteToDevice(dev, anchorCode.Address, anchorCode.Length, anchorCode.Data);
		
		if (kr != kIOReturnSuccess) { 
			NSLog(@"Error: WriteToDevice download returned err 0x%x", kr); 
			(*dev)->USBDeviceClose(dev); 
			(*dev)->Release(dev); 
			return kr; 
		}
		
		if ((anchorCode.Address % 2) == 0) {
			kr = ReadFromDevice(dev, anchorCode.Address, anchorCode.Length, anchorRead.Data);
			for (i = 0; i < anchorCode.Length; i++)
				if (anchorCode.Data[i] != anchorRead.Data[i])
					NSLog(@"Error: Data at 0x%X written: 0x%X read: 0x%X", anchorCode.Address + i, anchorCode.Data[i], anchorRead.Data[i]);
		} else {
			kr = ReadFromDevice(dev, anchorCode.Address, anchorCode.Length + 1, anchorRead.Data);
			for (i = 0; i < anchorCode.Length; i++)
				if (anchorCode.Data[i] != anchorRead.Data[i+1])
					NSLog(@"Error: Data at 0x%X written: 0x%X read: 0x%X", anchorCode.Address + i, anchorCode.Data[i], anchorRead.Data[i+1]);
		}		 
	}
	
	fclose(hexFile);
	
	// de-assert reset (tells device download complete)
	writeVal = 0;
	kr = WriteToDevice(dev, k8051_CPUCS, 1, &writeVal); 
	if (kr != kIOReturnSuccess) 
		NSLog(@"Error: WriteToDevice run returned err 0x%x", kr);
	
	return kr; 
}

/* 
* Read file into Intel Hex Record
 * TO DO: Implement checksum verification
 */
IOReturn hexRead(INTEL_HEX_RECORD *record, FILE *hexFile)
{
	char c;
	UInt16 i;
	int n, c1, check, len;
    c = getc(hexFile);
    
    if (c != ':') {
        NSLog(@"Error: Line does not start with colon (%d)", c);
        return kIOReturnNotAligned;
    }
	
    n = fscanf(hexFile, "%2lX%4lX%2lX", &record->Length, &record->Address, &record->Type);
	if (n != 3) {
        NSLog(@"Error: Could not read line preamble %d", c);
        return kIOReturnNotAligned;
    }
    
    len = record->Length;
    if (len > MAX_INTEL_HEX_RECORD_LENGTH) {
        NSLog(@"Error: length is more than can fit %d, %d", len, MAX_INTEL_HEX_RECORD_LENGTH);
        return kIOReturnNotAligned;
	}
    
	for (i = 0; i < len; i++) {
        n = fscanf(hexFile, "%2X", &c1);
        if (n != 1) {
            if (i != record->Length) {
                NSLog(@"Error: Line finished at wrong time %d, %ld", i, record->Length);
                return kIOReturnNotAligned;
            }
        }
        record->Data[i] = c1;
    }
    
	n = fscanf(hexFile, "%2X\n", &check);
    if (n != 1) {
		NSLog(@"Error: Check not found");
        return kIOReturnNotAligned;
    }
	
    return kIOReturnSuccess;
}

/*
 * Uses I/O Kit to create device interface for each raw device
 * Configures it, and downloads appropriate firmware
 */
void DeviceAdded(void *refCon, io_iterator_t iterator)
{
	kern_return_t               kr;
    io_service_t                usbDevice;
    IOCFPlugInInterface         **plugInInterface = NULL;
	IOUSBDeviceInterface        **dev = NULL;
    HRESULT                     result;
    SInt32                      score;
    UInt16                      vendor;
    UInt16                      product;
	
    while (usbDevice = IOIteratorNext(iterator)) {
		NSLog(@"Device Added...");
        // Create an intermediate plug-in
        kr = IOCreatePlugInInterfaceForService(usbDevice,
											   kIOUSBDeviceUserClientTypeID, kIOCFPlugInInterfaceID,
											   &plugInInterface, &score);
        // Don’t need the device object after intermediate plug-in is created
        kr = IOObjectRelease(usbDevice);
        if ((kIOReturnSuccess != kr) || !plugInInterface) {
            NSLog(@"Unable to create a plug-in (%08x)", kr);
            return;
        }
        // Now create the device interface
        result = (*plugInInterface)->QueryInterface(plugInInterface,
													CFUUIDGetUUIDBytes(kIOUSBDeviceInterfaceID),
													(LPVOID)&dev);
        // Don’t need the intermediate plug-in after device interface is created
        (*plugInInterface)->Release(plugInInterface);
		
        if (result || !dev) {
            NSLog(@"Could not create a device interface (%08x)", (int)result);
			return;
        }
		
        // Check these values for confirmation
        kr = (*dev)->GetDeviceVendor(dev, &vendor);
        kr = (*dev)->GetDeviceProduct(dev, &product);
        if ((vendor != kOurVendorID) || (product != kOurProductID)) {
            NSLog(@"Found unwanted device (vendor = 0x%x, product = 0x%x)", vendor, product);
            (void) (*dev)->Release(dev);
            return;
        }
		
        // Open the device to change its state
        kr = (*dev)->USBDeviceOpen(dev);
        if (kr != kIOReturnSuccess) {
            NSLog(@"Unable to open device: %08x", kr);
            (void) (*dev)->Release(dev);
            return;
        }
		
        // Configure device
        kr = ConfigureDevice(dev);
        if (kr != kIOReturnSuccess) {
            NSLog(@"Unable to configure device: %08x", kr);
            (void) (*dev)->USBDeviceClose(dev);
            (void) (*dev)->Release(dev);
            return;
        }
		
		if (fsm == 0) { // fpga_loader firmware
			NSLog(@"fsm = 0");
			NSLog(@"Downloading: %s", fnameloader);
			kr = DownloadToDevice(dev, fnameloader); 
			if (kr != kIOReturnSuccess) { 
				NSLog(@"Error: unable to download firmware to device: %08x", kr);
				(*dev)->USBDeviceClose(dev);
				(*dev)->Release(dev);
				return;
			}
			fsm++;
			kr = (*dev)->USBDeviceClose(dev); 
			kr = (*dev)->Release(dev);
			
		} else if (fsm == 1)  { // VHDL
			NSLog(@"fsm = 1");
			NSLog(@"Downloading: %s", fnamevhdl);
			kr = FindInterfaces(dev);
			if (kr != kIOReturnSuccess) {
				NSLog(@"Error: unable to find interfaces on device: %08x", kr);
				(*dev)->USBDeviceClose(dev);
				(*dev)->Release(dev);
				return;
			}
			fx2_fpga_sendxsvf(fnamevhdl);
			fsm++;
			kr = (*dev)->USBDeviceClose(dev); 
			kr = (*dev)->Release(dev);
			
		} else if (fsm == 2) { // driver firmware
			NSLog(@"fsm = 2");
			NSLog(@"Downloading: %s", fnamefirmware);
			kr = DownloadToDevice(dev, fnamefirmware);
			if (kr != kIOReturnSuccess) { 
				NSLog(@"Error: unable to download firmware to device: %08x", kr);
				(*dev)->USBDeviceClose(dev);
				(*dev)->Release(dev);
				return;
			}
			fsm++;
			kr = (*dev)->USBDeviceClose(dev); 
			kr = (*dev)->Release(dev);
			
		} else if (fsm == 3) { // read for bulk endpoint comm
			NSLog(@"fsm = 3");
			kr = FindInterfaces(dev);
			if (kr != kIOReturnSuccess) {
				NSLog(@"Error: unable to find interfaces on device: %08x", kr);
				(*dev)->USBDeviceClose(dev);
				(*dev)->Release(dev);
				continue;
			}
			fsm = 0; // device added now means unplugged (full reload needed)
			NSLog(@"***Device Ready***");
			ready = 1;
			[guiControl updateGUI:YES];
		}
	}
}

/*
 * Uses iterator to release each device object
 * This arms raw device termination notification
 */
void DeviceRemoved(void *refCon, io_iterator_t iterator)
{
	kern_return_t   kr;
    io_service_t    object;
	
    while (object = IOIteratorNext(iterator)) {
        NSLog(@"Device Removed...");
		kr = IOObjectRelease(object);
		
		curinterface = nil;
		downloadpipe = nil;
		uploadpipe = nil;
		
		ready = 0;
		[guiControl updateGUI:NO];
		
        if (kr != kIOReturnSuccess) {
            NSLog(@"Could not release raw device object: %08x", kr);
            continue;
        }
    }
}

/*
 * Sends protocol data to FPGA over USB
 */
int send_plotter(int command)
{
	IOReturn		kr;
	unsigned char	buf[8];
	
	if (command < 2) {
		buf[0] = command & 0x1;
		laser_state = command;
	} else {
		buf[0] = (command & 0xFE) | (laser_state & 0x1);
	}
	
	NSLog(@"Hex: %02X", buf[0]);
	if (curinterface == nil || downloadpipe == nil) {
		NSLog(@"SDD: No active interface or pipe");
		return 1;
	}
	
	kr = (*curinterface)->WritePipe(curinterface, 
									downloadpipe, 
									buf,
									1);
	if (kr != kIOReturnSuccess) {
		NSLog(@"SDD: Unable to perform bulk write (%08x)", kr);
		return 2;
	}
	
	NSLog(@"send_plotter okay");
	
	return 0;
}

/*
 * Used by XSVF parser
 */
int send_command(char buf[6])
{
	IOReturn	kr;
	
	kr = (*curinterface)->WritePipe(curinterface, 
									downloadpipe, 
									buf,
									6);
	if (kr != kIOReturnSuccess) {
		NSLog(@"SC: Unable to perform bulk write (%08x)", kr);
		return 1;
	}
		
	
	return 0;
}

/*
 * Used by XSVF parser
 */
int send_data(char data[], int bytes)
{
	IOReturn	kr;
	int			size = 0;
	int			done;
	
	done = 0;
	while (done < bytes) {
		if (bytes - done > 64)
			size = 64;
		else
			size = bytes - done;
		
		kr = (*curinterface)->WritePipe(curinterface, 
										downloadpipe, 
										data + done,
										size);
		if (kr != kIOReturnSuccess) {
			NSLog(@"SD: Unable to perform bulk write (%08x)", kr);
			return 1;
		}
		
		done += size;
	}
	
	return 0;
}

/*
 * Sends JTAG commands to the FPGA.
 * "filename" gives the location of an XSVF file.
 */
int fx2_fpga_sendxsvf(char *filename) 
{
	if (curinterface != nil) {
		// Run XSVF parser
		if (parse(filename))
			return 4;
		fx2_fpga_getsr();
		NSLog(@"Status Register: 0x%X", status);
		fx2_fpga_senddone();
	} else {
		NSLog(@"No USB Interface");
		return 1;
	}
	
	return 0;
}

/*
 * Reads the status register of the FPGA.
 * When done, "status" will have the value of the register.
 */
int fx2_fpga_getsr() 
{
	IOReturn	kr;
	char buffer[8];
	UInt32		numBytesRead;
	unsigned char cmd_readsr[] = {
		0x00, 0x00, 0x00, 0x00,
		0xAA, 0x99, 0x55, 0x66,
		0x28, 0x00, 0xE0, 0x01,
		0x00, 0x00, 0x00, 0x00
	};
	
	// Send command (CFG_IN)
	buffer[0] = 0x05;
	buffer[4] = 4;
	buffer[5] = 0;
	
	kr = (*curinterface)->WritePipe(curinterface, 
									downloadpipe, 
									buffer,
									6);
	if (kr != kIOReturnSuccess)
		NSLog(@"Unable to perform bulk write (%08x)", kr);
	
	// The data (read SR command)
	kr = (*curinterface)->WritePipe(curinterface, 
									downloadpipe, 
									cmd_readsr,
									sizeof(cmd_readsr));
	if (kr != kIOReturnSuccess)
		NSLog(@"Unable to perform bulk write (%08x)", kr);
	
	// Command (CFG_OUT)
	buffer[0] = 0x04;
	buffer[4] = 1;
	buffer[5] = 0;
	
	kr = (*curinterface)->WritePipe(curinterface, 
									downloadpipe, 
									buffer,
									6);
	if (kr != kIOReturnSuccess)
		NSLog(@"Unable to perform bulk write (%08x)", kr);
	
	// Read result
	buffer[0] = 0x05;
	buffer[4] = 4;
	buffer[5] = 0;
	
	numBytesRead = 4;
	kr = (*curinterface)->ReadPipe(curinterface, 
								   uploadpipe, 
								   buffer,
								   &numBytesRead);
	
	if (kr != kIOReturnSuccess)
		NSLog(@"Unable to perform bulk write (%08x)", kr);
	
	status = (buffer[2] << 16) | (buffer[1] << 8) | buffer[0];
	status = EndianU32_BtoN(status);
	
	return 0;
}

/*
 * Tells the fpga_loader firmware that we're done
 */
int fx2_fpga_senddone() 
{
	IOReturn	kr;
	char buffer[8];
	
	// Send command
	buffer[0] = (char)0xFF;
	
	kr = (*curinterface)->WritePipe(curinterface, 
									downloadpipe, 
									buffer,
									6);
	
	if (kr != kIOReturnSuccess)
		NSLog(@"Unable to perform bulk write (%08x)", kr);
	
	return 0;
}


/*
 * Creates device interface for each interface
 */
IOReturn FindInterfaces(IOUSBDeviceInterface **device)
{
    IOReturn                    kr;
    IOUSBFindInterfaceRequest   request;
    io_iterator_t               iterator;
    io_service_t                usbInterface;
    IOCFPlugInInterface         **plugInInterface = NULL;
    IOUSBInterfaceInterface     **interface = NULL;
    HRESULT                     result;
    SInt32                      score;
    UInt8                       interfaceNumEndpoints;
    int                         pipeRef;
	
    // Find all the interfaces
    request.bInterfaceClass = kIOUSBFindInterfaceDontCare;
    request.bInterfaceSubClass = kIOUSBFindInterfaceDontCare;
    request.bInterfaceProtocol = kIOUSBFindInterfaceDontCare;
    request.bAlternateSetting = kIOUSBFindInterfaceDontCare;
	
    //Get an iterator for the interfaces on the device
    kr = (*device)->CreateInterfaceIterator(device,
											&request, 
											&iterator);
    while (usbInterface = IOIteratorNext(iterator)) {
        // Create an intermediate plug-in
        kr = IOCreatePlugInInterfaceForService(usbInterface,
											   kIOUSBInterfaceUserClientTypeID,
											   kIOCFPlugInInterfaceID,
											   &plugInInterface, &score);
        // Release the usbInterface object after getting the plug-in
        kr = IOObjectRelease(usbInterface);
        if ((kr != kIOReturnSuccess) || !plugInInterface) {
            NSLog(@"Unable to create a plug-in (%08x)", kr);
            break;
        }
		
        // Now create the device interface for the interface
        result = (*plugInInterface)->QueryInterface(plugInInterface,
													CFUUIDGetUUIDBytes(kIOUSBInterfaceInterfaceID),
													(LPVOID) &interface);
        // No longer need the intermediate plug-in
        (*plugInInterface)->Release(plugInInterface);
		
        if (result || !interface) {
            NSLog(@"Could not create a device interface for the interface (%08x)", (int) result);
            break;
        }
		
        // Now open the interface, instantiates pipes
        kr = (*interface)->USBInterfaceOpen(interface);
        if (kr != kIOReturnSuccess) {
            NSLog(@"Unable to open interface (%08x)", kr);
            (void) (*interface)->Release(interface);
            break;
        }			
		
        // Get the number of endpoints associated with this interface
        kr = (*interface)->GetNumEndpoints(interface,
										   &interfaceNumEndpoints);
        if (kr != kIOReturnSuccess) {
            NSLog(@"Unable to get number of endpoints (%08x)", kr);
            (void) (*interface)->USBInterfaceClose(interface);
            (void) (*interface)->Release(interface);
            break;
        }
		
        NSLog(@"Endpoint Enumeration: %d endpoints", interfaceNumEndpoints);
        // Access each pipe in turn, starting with the pipe at index 1
        for (pipeRef = 1; pipeRef <= interfaceNumEndpoints; pipeRef++) {
            IOReturn        kr2;
            UInt8           direction;
            UInt8           number;
            UInt8           transferType;
            UInt16          maxPacketSize;
            UInt8           interval;
			
            kr2 = (*interface)->GetPipeProperties(interface,
												  pipeRef, 
												  &direction,
												  &number, 
												  &transferType,
												  &maxPacketSize, 
												  &interval);
			
			NSLog(@"pipeRef: %d direction: 0x%x, number: 0x%x transferType: 0x%x, maxPacketSize: 0x%x, interval: 0x%x", pipeRef, direction, number, transferType, maxPacketSize, interval); // DEBUG
			
			curinterface = interface;			
			if (direction == kUSBOut)
				downloadpipe = pipeRef;
			else if (direction == kUSBIn)
				uploadpipe = pipeRef;
			
            if (kr2 != kIOReturnSuccess)
                NSLog(@"Unable to get properties of pipe %d (%08x)", pipeRef, kr2);
        }
    }
    return kr;
}

void AcquireDevice()
{
	mach_port_t             masterPort;
    CFMutableDictionaryRef  matchingDict;
	CFRunLoopSourceRef		runLoopSource;
    kern_return_t           kr;
    SInt32                  usbVendor = kOurVendorID;
    SInt32                  usbProduct = kOurProductID;
	
    // Create a master port for communication with the I/O Kit
    kr = IOMasterPort(MACH_PORT_NULL, &masterPort);
    if (kr || !masterPort) {
        NSLog(@"ERR: Could not create a master I/O Kit port(%08x)", kr);
        return;
    }
    // Set up matching dictionary for class IOUSBDevice and its subclasses
    matchingDict = IOServiceMatching(kIOUSBDeviceClassName);
    if (!matchingDict) {
        NSLog(@"Could not create a USB matching dictionary");
        mach_port_deallocate(mach_task_self(), masterPort);
        return;
    }
	
    // Add the vendor and product IDs to the matching dictionary
    CFDictionarySetValue(matchingDict, CFSTR(kUSBVendorName),
						 CFNumberCreate(kCFAllocatorDefault,
										kCFNumberSInt32Type, &usbVendor));
    CFDictionarySetValue(matchingDict, CFSTR(kUSBProductName),
						 CFNumberCreate(kCFAllocatorDefault,
										kCFNumberSInt32Type, &usbProduct));
	
    // Set up asynchronous notifications 
    gNotifyPort = IONotificationPortCreate(masterPort);
    runLoopSource = IONotificationPortGetRunLoopSource(gNotifyPort);
    CFRunLoopAddSource(CFRunLoopGetCurrent(), runLoopSource, 
					   kCFRunLoopDefaultMode);
	
    // Retain additional dictionary reference
    matchingDict = (CFMutableDictionaryRef) CFRetain(matchingDict);
	
    // Set up notifications
    kr = IOServiceAddMatchingNotification(gNotifyPort,
										  kIOFirstMatchNotification, matchingDict,
										  DeviceAdded, NULL, &addedIter);
    // Iterate over set of matching devices to access already-present devices and to arm the notification 
    DeviceAdded(NULL, addedIter);

    // Notification of termination
    kr = IOServiceAddMatchingNotification(gNotifyPort,
										  kIOTerminatedNotification, matchingDict,
										  DeviceRemoved, NULL, &removedIter);
    // Iterate over set of matching devices to release each one and to arm the notification
    DeviceRemoved(NULL, removedIter);

    // Finished with master port
    mach_port_deallocate(mach_task_self(), masterPort);
    masterPort = 0;
}

@end
