;					Descriptors.asm
;		Originally by Erin Dunn and Ben Bozsa
;
;		Modified by Nathan Friess and Patrick McNeil
;		
;		Purpose: Defines USB descriptors
;
;		WARNING: The size of this data must be less than
;				 0x99 bytes!
;
		VENDOR_ID						= 0x04b4 ; Cypress
		PRODUCT_ID						= 0x8613 ; EZ-USB FX2
		PRODUCT_VERSION					= 0x0100 ; 1.0


		TYPE_DEVICE_DESCRIPTOR			= 1
		TYPE_CONFIGURATION_DESCRIPTOR	= 2
		TYPE_STRING_DESCRIPTOR			= 3
		TYPE_INTERFACE_DESCRIPTOR		= 4
		TYPE_ENDPOINT_DESCRIPTOR		= 5
		TYPE_HID_DESCRIPTOR				= 0x21

		USB_ENDPOINT_TYPE_CONTROL		= 0x00
		USB_ENDPOINT_TYPE_ISOCHRONOUS	= 0x01
		USB_ENDPOINT_TYPE_BULK			= 0x02
		USB_ENDPOINT_TYPE_INTERRUPT		= 0x03

		.area usb_descr_area(ABS)
		.org 0x66

_DeviceDescr::
	.db 18						; Descriptor length
	.db TYPE_DEVICE_DESCRIPTOR	; Decriptor type
	.dw 0002					; Specification Version (BCD)
	.db 0xFF					; Device class
	.db 00						; Device sub-class
	.db 00						; Device sub-sub-class
	.db 64						; Maximum packet size
	.db VENDOR_ID & 0xFF
	.db (VENDOR_ID >> 8) & 0xFF
	.db PRODUCT_ID & 0xFF
	.db (PRODUCT_ID >> 8) & 0xFF
	.db 0, 0
	.db 1						; Manufacturer string index
	.db 2						; Product string index
	.db 0						; Serial number string index
	.db 1						; Number of configurations

_DeviceQualifierDescr::
	.db 10						; Descriptor length
	.db 0x06					; Decriptor type
	.dw 0002					; Specification Version (BCD)
	.db 00						; Device class
	.db 00						; Device sub-class
	.db 00						; Device sub-sub-class
	.db 64						; Maximum packet size (EP0?)
	.db 1						; Number of configurations
	.db 0


_HighSpeedConfigDescr::   
	.db 9									; Descriptor length
	.db TYPE_CONFIGURATION_DESCRIPTOR		; Descriptor type
	.db (HighSpeedConfigDescrEnd-_HighSpeedConfigDescr) & 0xFF			; Total Length (LSB)
	.db ((HighSpeedConfigDescrEnd-_HighSpeedConfigDescr) >> 8) & 0xFF	; Total Length (MSB)
	.db 1									; Number of interfaces
	.db 1									; Configuration number
	.db 0									; Configuration string
	.db 0xa0								; Attributes (b7 - buspwr, b6 - selfpwr, b5 - rwu)
	.db 50									; Power requirement (div 2 ma)

; Interface Descriptor
	.db 9							; Descriptor length
	.db TYPE_INTERFACE_DESCRIPTOR	; Descriptor type
	.db 0							; Zero-based index of this interface
	.db 0							; Alternate setting
	.db 2							; Number of end points 
	.db 0xff						; Interface class
	.db 00							; Interface sub class
	.db 00							; Interface sub sub class
	.db 0							; Interface descriptor string index

; Endpoint Descriptor (EP1 OUT)
	.db 7							; Descriptor length
	.db TYPE_ENDPOINT_DESCRIPTOR	; Descriptor type
	.db 01							; Endpoint number, and direction
	.db USB_ENDPOINT_TYPE_BULK		; Endpoint type
	.db 0x40						; Max packet size (LSB)
	.db 00							; Max packect size (MSB)
	.db 00							; Polling interval

; Endpoint Descriptor (EP1 IN)
	.db 7							; Descriptor length
	.db TYPE_ENDPOINT_DESCRIPTOR	; Descriptor type
	.db 0x81						; Endpoint number, and direction
	.db USB_ENDPOINT_TYPE_BULK		; Endpoint type
	.db 0x40						; Max packet size (LSB)
	.db 00							; Max packect size (MSB)
	.db 00							; Polling interval

HighSpeedConfigDescrEnd:



_FullSpeedConfigDescr::   
	.db 9									; Descriptor length
	.db TYPE_CONFIGURATION_DESCRIPTOR		; Descriptor type
	.db (FullSpeedConfigDescrEnd-_FullSpeedConfigDescr) & 0xFF			; Total Length (LSB)
	.db ((FullSpeedConfigDescrEnd-_FullSpeedConfigDescr) >> 8) & 0xFF	; Total Length (MSB)
	.db 1									; Number of interfaces
	.db 1									; Configuration number
	.db 0									; Configuration string
	.db 0xa0								; Attributes (b7 - buspwr, b6 - selfpwr, b5 - rwu)
	.db 50									; Power requirement (div 2 ma)

; Interface Descriptor
	.db 9								; Descriptor length
	.db TYPE_INTERFACE_DESCRIPTOR		; Descriptor type
	.db 0								; Zero-based index of this interface
	.db 0								; Alternate setting
	.db 2								; Number of end points 
	.db 0xff							; Interface class
	.db 00								; Interface sub class
	.db 00								; Interface sub sub class
	.db 0								; Interface descriptor string index

; Endpoint Descriptor (EP1 OUT)
	.db 7							; Descriptor length
	.db TYPE_ENDPOINT_DESCRIPTOR	; Descriptor type
	.db 01							; Endpoint number, and direction
	.db USB_ENDPOINT_TYPE_BULK		; Endpoint type
	.db 0x40						; Max packet size (LSB)
	.db 00							; Max packet size (MSB)
	.db 00							; Polling interval

; Endpoint Descriptor (EP1 IN)
	.db 7							; Descriptor length
	.db TYPE_ENDPOINT_DESCRIPTOR	; Descriptor type
	.db 0x81						; Endpoint number, and direction
	.db USB_ENDPOINT_TYPE_BULK		; Endpoint type
	.db 0x40						; Max packet size (LSB)
	.db 00							; Max packet size (MSB)
	.db 00							; Polling interval

FullSpeedConfigDescrEnd:



_StringDscr0::
	.db StringDscr0End-_StringDscr0		; Len
	.db TYPE_STRING_DESCRIPTOR
	.db 0x09,04
StringDscr0End:

_StringDscr1::   
	.db StringDscr1End-_StringDscr1		; Len
	.db TYPE_STRING_DESCRIPTOR
	.db 'C,00
	.db 'y,00
	.db 'p,00
	.db 'r,00
	.db 'e,00
	.db 's,00
	.db 's,00
StringDscr1End:

_StringDscr2::   
	.db StringDscr2End-_StringDscr2		; Len
	.db TYPE_STRING_DESCRIPTOR
	.db 'E,00
	.db 'Z,00
	.db '-,00
	.db 'U,00
	.db 'S,00
	.db 'B,00
	.db ' ,00
	.db 'F,00
	.db 'X,00
	.db '2,00
StringDscr2End:
