/* DVB-T USB2 firmware for Cypress EZ-USB FX2
 * Cypress EZ-USB FX2 USB/GPIF register definitions
 */

#ifndef __FX2REGS_H__
#define __FX2REGS_H__

#ifdef ALLOCATE_REGISTERS
#define BYTE_REG(name, addr) xdata at addr volatile unsigned char name
#else
#define BYTE_REG(name, addr) extern xdata volatile unsigned char name
#endif

BYTE_REG(GPIF_WAVE_DATA, 0xE400);
BYTE_REG(RES_WAVEDATA_END, 0xE480);

/* General Configuration */

BYTE_REG(CPUCS, 0xE600);	/* Control & Status */
BYTE_REG(IFCONFIG, 0xE601);	/* Interface Configuration */
BYTE_REG(PINFLAGSAB, 0xE602);	/* FIFO FLAGA and FLAGB Assignments */
BYTE_REG(PINFLAGSCD, 0xE603);	/* FIFO FLAGC and FLAGD Assignments */
BYTE_REG(FIFORESET, 0xE604);	/* Restore FIFOS to default state */
BYTE_REG(BREAKPT, 0xE605);	/* Breakpoint */
BYTE_REG(BPADDRH, 0xE606);	/* Breakpoint Address H */
BYTE_REG(BPADDRL, 0xE607);	/* Breakpoint Address L */
BYTE_REG(UART230, 0xE608);	/* 230 Kbaud clock for T0,T1,T2 */
BYTE_REG(FIFOPINPOLAR, 0xE609);	/* FIFO polarities */
BYTE_REG(REVID, 0xE60A);	/* Chip Revision */
BYTE_REG(REVCTL, 0xE60B);	/* Chip Revision Control */

/* Endpoint Configuration */

BYTE_REG(EP1OUTCFG, 0xE610);
BYTE_REG(EP1INCFG, 0xE611);
BYTE_REG(EP2CFG, 0xE612);
BYTE_REG(EP4CFG, 0xE613);
BYTE_REG(EP6CFG, 0xE614);
BYTE_REG(EP8CFG, 0xE615);
BYTE_REG(EP2FIFOCFG, 0xE618);
BYTE_REG(EP4FIFOCFG, 0xE619);
BYTE_REG(EP6FIFOCFG, 0xE61A);
BYTE_REG(EP8FIFOCFG, 0xE61B);
BYTE_REG(EP2AUTOINLENH, 0xE620);
BYTE_REG(EP2AUTOINLENL, 0xE621);
BYTE_REG(EP4AUTOINLENH, 0xE622);
BYTE_REG(EP4AUTOINLENL, 0xE623);
BYTE_REG(EP6AUTOINLENH, 0xE624);
BYTE_REG(EP6AUTOINLENL, 0xE625);
BYTE_REG(EP8AUTOINLENH, 0xE626);
BYTE_REG(EP8AUTOINLENL, 0xE627);
BYTE_REG(EP2FIFOPFH, 0xE630);
BYTE_REG(EP2FIFOPFL, 0xE631);
BYTE_REG(EP4FIFOPFH, 0xE632);
BYTE_REG(EP4FIFOPFL, 0xE633);
BYTE_REG(EP6FIFOPFH, 0xE634);
BYTE_REG(EP6FIFOPFL, 0xE635);
BYTE_REG(EP8FIFOPFH, 0xE636);
BYTE_REG(EP8FIFOPFL, 0xE637);
BYTE_REG(EP2ISOINPKTS, 0xE640);
BYTE_REG(EP4ISOINPKTS, 0xE641);
BYTE_REG(EP6ISOINPKTS, 0xE642);
BYTE_REG(EP8ISOINPKTS, 0xE643);
BYTE_REG(INPKTEND, 0xE648);
BYTE_REG(OUTPKTEND, 0xE649);

/* Interrupts */

BYTE_REG(EP2FIFOIE, 0xE650);	/* Endpoint 2 Flag Interrupt Enable */
BYTE_REG(EP2FIFOIRQ, 0xE651);	/* Endpoint 2 Flag Interrupt Request */
BYTE_REG(EP4FIFOIE, 0xE652);	/* Endpoint 4 Flag Interrupt Enable */
BYTE_REG(EP4FIFOIRQ, 0xE653);	/* Endpoint 4 Flag Interrupt Request */
BYTE_REG(EP6FIFOIE, 0xE654);	/* Endpoint 6 Flag Interrupt Enable */
BYTE_REG(EP6FIFOIRQ, 0xE655);	/* Endpoint 6 Flag Interrupt Request */
BYTE_REG(EP8FIFOIE, 0xE656);	/* Endpoint 8 Flag Interrupt Enable */
BYTE_REG(EP8FIFOIRQ, 0xE657);	/* Endpoint 8 Flag Interrupt Request */
BYTE_REG(IBNIE, 0xE658);	/* IN-BULK-NAK Interrupt Enable */
BYTE_REG(IBNIRQ, 0xE659);	/* IN-BULK-NAK interrupt Request */
BYTE_REG(NAKIE, 0xE65A);	/* Endpoint Ping NAK interrupt Enable */
BYTE_REG(NAKIRQ, 0xE65B);	/* Endpoint Ping NAK interrupt Request */
BYTE_REG(USBIE, 0xE65C);	/* USB Int Enables */
BYTE_REG(USBIRQ, 0xE65D);	/* USB Interrupt Requests */
BYTE_REG(EPIE, 0xE65E);		/* Endpoint Interrupt Enables */
BYTE_REG(EPIRQ, 0xE65F);	/* Endpoint Interrupt Requests */
BYTE_REG(GPIFIE, 0xE660);	/* GPIF Interrupt Enable */
BYTE_REG(GPIFIRQ, 0xE661);	/* GPIF Interrupt Request */
BYTE_REG(USBERRIE, 0xE662);	/* USB Error Interrupt Enables */
BYTE_REG(USBERRIRQ, 0xE663);	/* USB Error Interrupt Requests */
BYTE_REG(ERRCNTLIM, 0xE664);	/* USB Error counter and limit */
BYTE_REG(CLRERRCNT, 0xE665);	/* Clear Error Counter EC[3..0] */
BYTE_REG(INT2IVEC, 0xE666);	/* Interupt 2 (USB) Autovector */
BYTE_REG(INT4IVEC, 0xE667);	/* Interupt 4 (FIFOS & GPIF) Autovector */
BYTE_REG(INTSETUP, 0xE668);	/* Interrupt 2&4 Setup */

/* Input/Output */

BYTE_REG(PORTACFG, 0xE670);	/* I/O PORTA Alternate Configuration */
BYTE_REG(PORTCCFG, 0xE671);	/* I/O PORTC Alternate Configuration */
BYTE_REG(PORTECFG, 0xE672);	/* I/O PORTE Alternate Configuration */
BYTE_REG(I2CS, 0xE678);		/* Control & Status */
BYTE_REG(I2DAT, 0xE679);	/* Data */
BYTE_REG(I2CTL, 0xE67A);	/* I2C Control */
BYTE_REG(XAUTODAT1, 0xE67B);	/* Autoptr1 MOVX access */
BYTE_REG(XAUTODAT2, 0xE67C);	/* Autoptr2 MOVX access */

/* USB Control */

BYTE_REG(USBCS, 0xE680);	/* USB Control & Status */
BYTE_REG(SUSPEND, 0xE681);	/* Put chip into suspend */
BYTE_REG(WAKEUPCS, 0xE682);	/* Wakeup source and polarity */
BYTE_REG(TOGCTL, 0xE683);	/* Toggle Control */
BYTE_REG(USBFRAMEH, 0xE684);	/* USB Frame count H */
BYTE_REG(USBFRAMEL, 0xE685);	/* USB Frame count L */
BYTE_REG(MICROFRAME, 0xE686);	/* Microframe count, 0-7 */
BYTE_REG(FNADDR, 0xE687);	/* USB Function address */

/* Endpoints */

BYTE_REG(EP0BCH, 0xE68A);	/* Endpoint 0 Byte Count H */
BYTE_REG(EP0BCL, 0xE68B);	/* Endpoint 0 Byte Count L */
BYTE_REG(EP1OUTBC, 0xE68D);	/* Endpoint 1 OUT Byte Count */
BYTE_REG(EP1INBC, 0xE68F);	/* Endpoint 1 IN Byte Count */
BYTE_REG(EP2BCH, 0xE690);	/* Endpoint 2 Byte Count H */
BYTE_REG(EP2BCL, 0xE691);	/* Endpoint 2 Byte Count L */
BYTE_REG(EP4BCH, 0xE694);	/* Endpoint 4 Byte Count H */
BYTE_REG(EP4BCL, 0xE695);	/* Endpoint 4 Byte Count L */
BYTE_REG(EP6BCH, 0xE698);	/* Endpoint 6 Byte Count H */
BYTE_REG(EP6BCL, 0xE699);	/* Endpoint 6 Byte Count L */
BYTE_REG(EP8BCH, 0xE69C);	/* Endpoint 8 Byte Count H */
BYTE_REG(EP8BCL, 0xE69D);	/* Endpoint 8 Byte Count L */
BYTE_REG(EP0CS, 0xE6A0);	/* Endpoint  Control and Status */
BYTE_REG(EP1OUTCS, 0xE6A1);	/* Endpoint 1 OUT Control and Status */
BYTE_REG(EP1INCS, 0xE6A2);	/* Endpoint 1 IN Control and Status */
BYTE_REG(EP2CS, 0xE6A3);	/* Endpoint 2 Control and Status */
BYTE_REG(EP4CS, 0xE6A4);	/* Endpoint 4 Control and Status */
BYTE_REG(EP6CS, 0xE6A5);	/* Endpoint 6 Control and Status */
BYTE_REG(EP8CS, 0xE6A6);	/* Endpoint 8 Control and Status */
BYTE_REG(EP2FIFOFLGS, 0xE6A7);	/* Endpoint 2 Flags */
BYTE_REG(EP4FIFOFLGS, 0xE6A8);	/* Endpoint 4 Flags */
BYTE_REG(EP6FIFOFLGS, 0xE6A9);	/* Endpoint 6 Flags */
BYTE_REG(EP8FIFOFLGS, 0xE6AA);	/* Endpoint 8 Flags */
BYTE_REG(EP2FIFOBCH, 0xE6AB);	/* EP2 FIFO total byte count H */
BYTE_REG(EP2FIFOBCL, 0xE6AC);	/* EP2 FIFO total byte count L */
BYTE_REG(EP4FIFOBCH, 0xE6AD);	/* EP4 FIFO total byte count H */
BYTE_REG(EP4FIFOBCL, 0xE6AE);	/* EP4 FIFO total byte count L */
BYTE_REG(EP6FIFOBCH, 0xE6AF);	/* EP6 FIFO total byte count H */
BYTE_REG(EP6FIFOBCL, 0xE6B0);	/* EP6 FIFO total byte count L */
BYTE_REG(EP8FIFOBCH, 0xE6B1);	/* EP8 FIFO total byte count H */
BYTE_REG(EP8FIFOBCL, 0xE6B2);	/* EP8 FIFO total byte count L */
BYTE_REG(SUDPTRH, 0xE6B3);	/* Setup Data Pointer high address byte */
BYTE_REG(SUDPTRL, 0xE6B4);	/* Setup Data Pointer low address byte */
BYTE_REG(SUDPTRCTL, 0xE6B5);	/* Setup Data Pointer Auto Mode */
BYTE_REG(SETUPDAT[8], 0xE6B8);	/* 8 bytes of SETUP data */

/* GPIF */

BYTE_REG(GPIFWFSELECT, 0xE6C0);	/* Waveform Selector */
BYTE_REG(GPIFIDLECS, 0xE6C1);	/* GPIF Done, GPIF IDLE drive mode */
BYTE_REG(GPIFIDLECTL, 0xE6C2);	/* Inactive Bus, CTL states */
BYTE_REG(GPIFCTLCFG, 0xE6C3);	/* CTL OUT pin drive */
BYTE_REG(GPIFADRH, 0xE6C4);	/* GPIF Address H */
BYTE_REG(GPIFADRL, 0xE6C5);	/* GPIF Address L */

BYTE_REG(GPIFTCB3, 0xE6CE);	/* GPIF Transaction Count Byte 3 */
BYTE_REG(GPIFTCB2, 0xE6CF);	/* GPIF Transaction Count Byte 2 */
BYTE_REG(GPIFTCB1, 0xE6D0);	/* GPIF Transaction Count Byte 1 */
BYTE_REG(GPIFTCB0, 0xE6D1);	/* GPIF Transaction Count Byte 0 */

BYTE_REG(EP2GPIFFLGSEL, 0xE6D2);	/* EP2 GPIF Flag select */
BYTE_REG(EP2GPIFPFSTOP, 0xE6D3);	/* Stop GPIF EP2 transaction on prog. flag */
BYTE_REG(EP2GPIFTRIG, 0xE6D4);		/* EP2 FIFO Trigger */
BYTE_REG(EP4GPIFFLGSEL, 0xE6DA);	/* EP4 GPIF Flag select */
BYTE_REG(EP4GPIFPFSTOP, 0xE6DB);	/* Stop GPIF EP4 transaction on prog. flag */
BYTE_REG(EP4GPIFTRIG, 0xE6DC);		/* EP4 FIFO Trigger */
BYTE_REG(EP6GPIFFLGSEL, 0xE6E2);	/* EP6 GPIF Flag select */
BYTE_REG(EP6GPIFPFSTOP, 0xE6E3);	/* Stop GPIF EP6 transaction on prog. flag */
BYTE_REG(EP6GPIFTRIG, 0xE6E4);		/* EP6 FIFO Trigger */
BYTE_REG(EP8GPIFFLGSEL, 0xE6EA);	/* EP8 GPIF Flag select */
BYTE_REG(EP8GPIFPFSTOP, 0xE6EB);	/* Stop GPIF EP8 transaction on prog. flag */
BYTE_REG(EP8GPIFTRIG, 0xE6EC);		/* EP8 FIFO Trigger */
BYTE_REG(XGPIFSGLDATH, 0xE6F0);		/* GPIF Data H (16-bit mode only) */
BYTE_REG(XGPIFSGLDATLX, 0xE6F1);	/* Read/Write GPIF Data L & trigger transac */
BYTE_REG(XGPIFSGLDATLNOX, 0xE6F2);	/* Read GPIF Data L, no transac trigger */
BYTE_REG(GPIFREADYCFG, 0xE6F3);		/* Internal RDY,Sync/Async, RDY5CFG */
BYTE_REG(GPIFREADYSTAT, 0xE6F4);	/* RDY pin states */
BYTE_REG(GPIFABORT, 0xE6F5);		/* Abort GPIF cycles */

/* UDMA */

BYTE_REG(FLOWSTATE, 0xE6C6);	/*Defines GPIF flow state */
BYTE_REG(FLOWLOGIC, 0xE6C7);	/*Defines flow/hold decision criteria */
BYTE_REG(FLOWEQ0CTL, 0xE6C8);	/*CTL states during active flow state */
BYTE_REG(FLOWEQ1CTL, 0xE6C9);	/*CTL states during hold flow state */
BYTE_REG(FLOWHOLDOFF, 0xE6CA);
BYTE_REG(FLOWSTB, 0xE6CB);	/*CTL/RDY Signal to use as master data strobe  */
BYTE_REG(FLOWSTBEDGE, 0xE6CC);	/*Defines active master strobe edge */
BYTE_REG(FLOWSTBHPERIOD, 0xE6CD);	/*Half Period of output master strobe */
BYTE_REG(GPIFHOLDAMOUNT, 0xE60C);	/*Data delay shift  */
BYTE_REG(UDMACRCH, 0xE67D);	/*CRC Upper byte */
BYTE_REG(UDMACRCL, 0xE67E);	/*CRC Lower byte */
BYTE_REG(UDMACRCQUAL, 0xE67F);	/*UDMA In only, host terminated use only */


/* Debug/Test */

BYTE_REG(DBUG, 0xE6F8);		/* Debug */
BYTE_REG(TESTCFG, 0xE6F9);	/* Test configuration */
BYTE_REG(USBTEST, 0xE6FA);	/* USB Test Modes */
BYTE_REG(CT1, 0xE6FB);		/* Chirp Test--Override */
BYTE_REG(CT2, 0xE6FC);		/* Chirp Test--FSM */
BYTE_REG(CT3, 0xE6FD);		/* Chirp Test--Control Signals */
BYTE_REG(CT4, 0xE6FE);		/* Chirp Test--Inputs */

/* Endpoint Buffers */

BYTE_REG(EP0BUF[64], 0xE740);	/* EP0 IN-OUT buffer */
BYTE_REG(EP1OUTBUF[64], 0xE780);	/* EP1-OUT buffer */
BYTE_REG(EP1INBUF[64], 0xE7C0);		/* EP1-IN buffer */
BYTE_REG(EP2FIFOBUF[1024], 0xF000);	/* 512/1024-byte EP2 buffer (IN or OUT) */
BYTE_REG(EP4FIFOBUF[1024], 0xF400);	/* 512 byte EP4 buffer (IN or OUT) */
BYTE_REG(EP6FIFOBUF[1024], 0xF800);	/* 512/1024-byte EP6 buffer (IN or OUT) */
BYTE_REG(EP8FIFOBUF[1024], 0xFC00);	/* 512 byte EP8 buffer (IN or OUT) */

#undef BYTE_REG


#endif /* __FX2REGS_H__ */
