/*
 * plottest, main.c
 * Programmer: Andrew Stagg
 * Last Modified: October 15, 2007
 * Based on: usbskel
 * By Nathan Friess and Patrick McNeil
 * April 12, 2005
 *
 * The entry point for the firmware.
 */

#include "fx2.h"
#include "fx2regs.h"

#define INT_TMR0	1
#define INTS_ENABLE 0x80
#define INT_TMR0_EN 0x02
#define TMR_EN0		0x10

extern volatile unsigned char ep1_out_rdy;
extern volatile unsigned char ep1_in_rdy;

// LED definitions
sbit at 0x93 EP1_OUT_LED;
sbit at 0x92 EP1_IN_LED;

extern void sleep(unsigned int);

/*
 * Waits for data to arrive on endpoint 1.
 * While waiting, blinks an LED so the user knows what's going.
 *
 * Returns the number of bytes received from USB host.
 * (data will be in EP1OUTBUF)
 */
unsigned char ep1_wait_for_data() {
        unsigned long i;
	unsigned char c = 0;

	ep1_out_rdy = 0;
	EP1OUTBC = 1; // Arm OUT buffer

top_ep1_out:

	// Wait for data to arrive
	while (!ep1_out_rdy) {
	  for (i = 0; i < 70000; i++) {
			if (ep1_out_rdy)
				goto done_ep1_out;
		}

		if (c) {
			EP1_OUT_LED = 1;
			c = 0;
		}
		else {
			EP1_OUT_LED = 0;
			c = 1;
		}
	}
done_ep1_out:
	if (EP1OUTCS & 0x2) {
		// False interrupt...
		ep1_out_rdy = 0;
		goto top_ep1_out;
	}

	EP1_OUT_LED = 1;

	c = EP1OUTBC;

	return c;
}

/*
 * Sends data to USB host on endpoint 1.
 * While waiting for acknowledgement, blinks an LED so the user knows what's going.
 *
 * len: The number of bytes to send (data should already be loaded in EP1INBUF).
 */
void ep1_send_data(unsigned char len) {
	unsigned long i;
	unsigned char c = 0;

	ep1_in_rdy = 0;
	EP1INBC = len; // Start transfer

	// Wait for data to be sent
	while (!ep1_in_rdy) {
		for (i = 0; i < 70000; i++) {
			if (ep1_in_rdy)
				goto done_ep1_in;
		}

		if (c) {
			EP1_IN_LED = 1;
			c = 0;
		}
		else {
			EP1_IN_LED = 0;
			c = 1;
		}
	}
done_ep1_in:

	EP1_IN_LED = 1;
}

// The entry point.
void main(void) {
  unsigned char cmd = 0x00;
  
  // Init USB
  EUSB = 1; // Enable USB interrupt
  
  USBIE |= 0xFF;  // Enable all USB Control interrupts
  
  INTSETUP = 0x08; // Enable USB Autovectoring
  
  EPIE |= 0x0F; // Enable EP0/EP1 interrupts
  
  // Init CPU clock, 48mhz
  CPUCS = 0x10;
  
  // Set SRAM to as fast as possible
  CKCON = 0;
  
  // Init interrupts
  IE = INTS_ENABLE;
  
  sleep(500);
  
  USBCS |= 0x08; // DISCONNECT
  USBCS |= 0x02; // RENUM
  
  // delay for 500ms ?  I've even seen 1500 ms
  sleep(500);
  
  // maybe we should even clear IRQs
  
  USBCS &= ~0x08; // RECONNECT
  
  // delay again...
  sleep(500);
  
  // CLKOUT enabled
  CPUCS |= 0x02;
  
  OEC = 0x3D; /* 0011 1101 */
  OEB = 0xFF; /* 1111 1111 */  
  
  while (1) {
    IOB = 0xFF;
    ep1_wait_for_data();
    cmd = EP1OUTBUF[0];
    cmd &= 0x3D;
    IOC = cmd;
    IOB = cmd ^ 0xFF; /* invert */
    IOC = cmd & 0x1; /* re-arm */
  }
}
