; usb-skel, sleep.asm
;
; By Nathan Friess and Patrick McNeil
; April 12, 2005
;
; A delay function, to make the firmware wait for a period of time.
; The length of time is given in milliseconds.
;
; Tested with an oscilloscope to verify general accuracy.

.AREA CSEG

; In theory:
; (48 Mhz in ms) / (inner loop)) - ((outer loop) / (inner loop)), approx
; (does not include time for call/return, and for saving delayms... a constant 12 clocks)
; In reality, it was 4 times too slow, hence the 12000 instead of 480000.
COUNT = ((12000/20)) - 2

; C declaration:
; void sleep(unsigned int delayms);
_sleep::

; delayms is in DPL, DPH

; All of the "_delay" labels are used to ensure that all branch paths
; take the same number of clock cycles.

;	save delayms						## 4
	mov		r0, dph
	mov		r1, dpl

; do {									## BEGIN OUTER LOOP
delayms_top:


;   unsigned int i = COUNT;				## 4
	mov		r2, #((COUNT >> 8) & 0xFF)
	mov		r3, #(COUNT & 0xFF)

;   do {								## BEGIN INNER LOOP
i_top:

;										## 2
	mov		r4, #0x00

;     i--;								## 10
	dec	r3
	cjne	r3, #0xFF, i_skip_borrow_delay
	dec	r2
	ljmp	i_skip_borrow
i_skip_borrow_delay:
	nop
	ljmp	i_skip_borrow	
i_skip_borrow:

;   } while (i != 0)					## 8
	cjne	r2, #0x00, i_top_delay
	cjne	r3, #0x00, i_top
;										## END INNER LOOP, TOTAL 20


; delayms--;							## 10
	dec	r1
	cjne	r1, #0xFF, delayms_skip_borrow_delay
	dec	r0
	ljmp	delayms_skip_borrow
delayms_skip_borrow_delay:
	nop
	ljmp	delayms_skip_borrow	
delayms_skip_borrow:


;	align outer loop to multiple of inner loop		## 18
	mov		r4, #0x00
	mov		r4, #0x00
	mov		r4, #0x00
	mov		r4, #0x00
	mov		r4, #0x00
	mov		r4, #0x00
	mov		r4, #0x00
	mov		r4, #0x00
	mov		r4, #0x00


; } while (delayms != 0);				## 8
	cjne	r0, #0x00, delayms_top_delay
	cjne	r1, #0x00, delayms_top
;										## END OUTER LOOP, TOTAL 40

; All done!
	ret

; Dummy branches to fill time.
i_top_delay:
	ljmp	i_top

delayms_top_delay:
	ljmp	delayms_top
